// CS250_A3 Framework.cpp : Takes command line args stack#, slice#, radius, and output file name and generates
// a sphere model using spherical coords.

#include "stdafx.h"

using namespace std;

#define PI		3.1415926535897932384626433832795
#define DEBUG	false

typedef struct Vertex
{
	float x,y,z;
}Vertex;

typedef struct Triangle
{
	int v0,v1,v2;
}Triangle;

int main(int argc, char* argv[])
{
	//in the case that the user entered incorrect args to the command line
	if(argc != 5 || strtol(argv[1], NULL, 10) < 1 || strtol(argv[2], NULL, 10) < 1 || strtol(argv[3], NULL, 10) < 1 || sizeof(argv[4]) < 1)
	{
		cout<<"Usage: sphere radius stacks slices filename[.ifs]"<<endl;
		return 0;
	}
	
	vector<Vertex> vertList;
	vector<Triangle> triList;
	Vertex curVert;
	string modelName(argv[4]);
	unsigned int radius, stacks, slices;
	radius = strtol(argv[1], NULL, 10);
	stacks = strtol(argv[2], NULL, 10);
	slices = strtol(argv[3], NULL, 10);

	float theta, phi, curTheta, curPhi;
	theta = static_cast<float>(PI/stacks);		//our angle increment for slicing goes from -PI/2 to PI/2
	phi = static_cast<float>((2*PI)/slices);	//our angle inrement for stacks goes from 0 to 2PI

	if(DEBUG)
	{
		cout<<"Radius: "<<radius<<"\nStacks: "<<stacks<<"\nSlices: "<<slices<<"\nModel Name: "<<modelName.c_str()
		<<"\nLength of model name: "<< modelName.length() + 1 <<"\nAngle Theta: "<<theta<<"\nAngle Phi: "<< phi << endl;
	}

	curTheta = static_cast<float>(PI/2);			////////////////////////////////////////////////////////					
	curVert.x = 0;									//Setting up our start point for getting our vertices.
	curVert.y = 0;									//We start at the very top of the circle, so we get the
	curVert.z = static_cast<float>(radius);			//vertex at the top point and add it to the list.
	vertList.push_back(curVert);					//////////////////////////////////////////////////////

	for(unsigned int stack = 0; stack < stacks - 1; stack++)  //calculate all vertices
	{
		curPhi = 0;
		curTheta -= theta;
		for(unsigned int slice = 0; slice < slices; slice++)
		{
			curVert.x = radius * cos(curTheta) * cos(curPhi);
			curVert.y = radius * cos(curTheta) * sin(curPhi);
			curVert.z = radius * sin(curTheta);
			vertList.push_back(curVert);
			curPhi += phi;
		}
	}

	curVert.x = 0;									/////////////////////////////////////////////////////////////
	curVert.y = 0;									//Add the vertex at the very bottom of the sphere to the list
	curVert.z = -static_cast<float>(radius);		//////////////////////////////////////////////////////////////
	vertList.push_back(curVert);

	if(vertList.size() < 3 || slices < 3)  //don't waste time with no triangles and no 3D
	{
		cout<<"Not a valid sphere."<<endl;
		return 0;
	}
	
	//triangle algorithm for top stack
	Triangle t;
	unsigned int mul = 0, vSize = static_cast<unsigned int>(vertList.size()) - 1;
	for(unsigned int tri = 1; tri <= slices; tri++)
	{
		t.v0 = 0;
		t.v1 = tri;
		t.v2 = (tri%slices) + 1;
		triList.push_back(t);
	}

	//triangle algorithm for middle stacks
	//first tri: (num, num + slices, (num%slices) + slices + 1)
	//second tri: (num, (num%slices) + slices + 1, (num%slices) + 1)
	for(unsigned int stack = 0; stack < stacks-2; stack++, mul++ )
	{
		for(unsigned int vertTrack = 1; vertTrack <= slices; vertTrack++ )
		{
			//fisrt tri
			t.v0 = (mul*stacks) + vertTrack;
			t.v1 = (mul*stacks) + vertTrack + slices;
			t.v2 = (mul*stacks) + (vertTrack % slices) + slices + 1;
			triList.push_back(t);

			//second tri
			t.v0 = (mul*stacks) + vertTrack;
			t.v1 = (mul*stacks) + (vertTrack % slices) + slices + 1;
			t.v2 = (mul*stacks) + (vertTrack % slices) + 1;
			triList.push_back(t);
		}
	}

	//triangle algorithm for bottom stack
	for(unsigned int tri = vSize - slices; tri < vSize - 1; tri++)
	{
		t.v0 = vSize;
		t.v1 = tri;
		t.v2 = tri + 1;
		triList.push_back(t);
	}
	t.v0 = vSize;                   ///////////////////////////
	t.v1 = vSize - 1;				//Add very last triangle
	t.v2 = vSize - slices;			///////////////////////////
	triList.push_back(t);

	if(DEBUG)
	{
		system("pause");
		cout<<"Vertex List: "<< endl;
		for(unsigned int i = 0; i < vertList.size(); i++)
			cout<<"x: "<<vertList[i].x<<" y: "<<vertList[i].y<<" z: "<<vertList[i].z<<endl<<endl;
		cout<<"Triangle List: "<< endl;
		for(unsigned int i = 0; i < triList.size(); i++)
			cout<<"v0: "<<triList[i].v0<<" v1: "<<triList[i].v1<<" v2: "<<triList[i].v2<<endl;
	}

	cout<<"\nStacks: "<<stacks<<" Slices: "<<slices<<" Vertices: "<<vertList.size()
		<<" Triangles: "<<triList.size()<<endl<<endl;


	return 0;
}

